####
## Code for submitted manuscript "An approximate Bayesian computation approach to parameter estimation in a stochastic stage structured population model"
## Katherine Scranton, Jonas Knape, Perry de Valpine
## 10/7/2013
## katherine.scranton@yale.edu

simParticleC <- function(theta, initN=rep(0,10), Tend=10, maxInd=300, sampT=seq(0,10,by=2)){

	#thetaC<-data.frame(matrix(c(4,3,0.7,5),1,4))
	#names(thetaC)<-c("eggscale","immscale","mort","fec")

	eggCounts <- numeric(length(sampT))
	immCounts <- numeric(length(sampT))
	adCounts <- numeric(length(sampT))
	
	birthTime <- numeric(maxInd)
	immTrans <- numeric(maxInd)
	adTrans <- numeric(maxInd)
	deathTime <- numeric(maxInd)

	eggShape <- 6
	immShape <- 6
	
	iList <- 1:maxInd
	totalInd <- length(initN)
	
	birthTime[1:totalInd] <- initN
	
	for (iPoint in iList){
		
		if (iPoint <= totalInd){
			
			immTrans[iPoint] <- birthTime[iPoint] + rweibull(1,shape=eggShape, scale=theta$eggscale)
			adTrans[iPoint] <- immTrans[iPoint] + rweibull(1,shape=immShape, scale=theta$immscale)
			deathTime[iPoint] <- adTrans[iPoint] + rexp(1,rate=theta$mort)
		
			eggCounts <- eggCounts + as.numeric((birthTime[iPoint]<=sampT)&(immTrans[iPoint]>sampT))
			immCounts <- immCounts + as.numeric((immTrans[iPoint]<=sampT)&(adTrans[iPoint]>sampT))
			adCounts <- adCounts + as.numeric((adTrans[iPoint]<=sampT)&(deathTime[iPoint]>sampT))
		
			if (adTrans[iPoint] < Tend){
			
				numOff <- floor( theta$fec*min( (deathTime[iPoint]-adTrans[iPoint]), (Tend-adTrans[iPoint])) )
							
				for (j in 1:numOff){
				
					if (totalInd < maxInd){
						birthTime[totalInd+1] <- adTrans[iPoint]+j/theta$fec
						totalInd = totalInd+1
					}				
				}			
			}
		}			
	}
	
	as.double(cbind(eggCounts,immCounts,adCounts))	
}

passParticletoC <- function(thetaC, initN=rep(0,10), Tend=10, maxInd=450, sampT=seq(0,10,by=2)){
	
	#ans<-simParticleC(thetaC,initN,Tend,maxInd,sampT)
	
	if (length(sampT) < 1){
		stop("sampT must be non-empty")
	}
	
	# theta should be organized as:
	# (eggShape, sggScale, immShape,immScale,mortality,fecundity)
	eggShape <- 6
	immShape <- 6
	
	theta <- as.double(c(eggShape, thetaC[1], immShape, thetaC[2:4]))
	
	if (length(theta)%%2==1){
		stop("incorrect parameter specification")
	}
	
	nStages <- (length(theta)-2)/2 +1
	inds <- length(initN)
	
	ans <- .C("rCohort",as.double(sampT), as.integer(length(sampT)), as.double(theta), as.integer(nStages), as.integer(maxInd), as.integer(inds), integer(nStages*length(sampT)))
	
	as.double(ans[[7]])
}